@@ -16,7 +16,24 @@ class UDFMaster(object):
         model_servers = RayContext.parse_servers(conf["modelServers"])
         items = RayContext.collect_from(model_servers)
         model_refs = [ray.put(item) for item in items]
-        self.actors = dict([(index, UDFWorker.remote(model_refs, conf, init_func, apply_func)) for index in range(num)])
+
+        udfWorkerConf = {}
+
+        if "num_cpus" in conf:
+            udfWorkerConf["num_cpus"] = int(conf["num_cpus"])
+
+        if "num_gpus" in conf:
+            udfWorkerConf["num_gpus"] = int(conf["num_gpus"])
+
+        custom_resources = [(key.split("resource.")[1], float(conf[key])) for key in
+                            conf.keys() if
+                            key.startswith("resource.")]
+
+        if len(custom_resources) > 0:
+            udfWorkerConf["resources"] = dict(custom_resources)
+
+        self.actors = dict(
+            [(index, UDFWorker.options(**udfWorkerConf).remote(model_refs, conf, init_func, apply_func)) for index in range(num)])
         self._idle_actors = [index for index in range(num)]
 
     def get(self) -> List[Any]:
@@ -59,9 +76,18 @@ class UDFBuilder(object):
         max_concurrency = int(conf.get("maxConcurrency", "3"))
 
         try:
+            stop_flag = True
+            counter = 30
             temp_udf_master = ray.get_actor(udf_name)
             ray.kill(temp_udf_master)
-            time.sleep(1)
+            while stop_flag or counter < 0:
+                time.sleep(1)
+                try:
+                    ray.get_actor(udf_name)
+                except Exception:
+                    stop_flag = False
+                    counter = counter - 1
+
         except Exception as inst:
             print(inst)
             pass
@@ -76,9 +102,9 @@ class UDFBuilder(object):
         udf_name = conf["UDF_CLIENT"]
         udf_master = ray.get_actor(udf_name)
         [index, worker] = ray.get(udf_master.get.remote())
-        input = [row["value"] for row in ray_context.python_context.fetch_once_as_rows()]
+        input_value = [row["value"] for row in ray_context.python_context.fetch_once_as_rows()]
         try:
-            res = ray.get(worker.apply.remote(input))
+            res = ray.get(worker.apply.remote(input_value))
         except Exception as inst:
             res = []
             print(inst)
